/* eslint-disable no-console */
import {
  Http, Metadata, Router,
} from '@zeedhi/core';
import {
  AlertService, Button, Chip, DialogService, Form, LoadingService, Row, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class ControllerNovoChecklist {
  private dadosOriginal: any = {};

  public chipTelaEdit = ({ component }: any) => { component.outlined = (window.location.search.slice(1) !== ''); };

  public setTipoLocal = (component: any) => { if (window.location.search.slice(1) === '') this.getTipoLocal(component); };

  public setItens = (component: any) => { if (window.location.search.slice(1) === '') this.getItens(component); };

  public Tarefa = (component: any) => { if (window.location.search.slice(1) === '') this.getTarefa(component); };

  public async getTipoLocal({ component }:any) {
    await Http.get('/getLocalChk').then((response: any) => {
      component.datasource.updateData(response.data.dataset.local);
    });
  }

  public async getItens({ component }:any) {
    await Http.get('/filtroItens').then((response: any) => {
      component.datasource.updateData(response.data.dataset.filtro);
    });
  }

  public async getTarefa({ component }:any) {
    await Http.get('/filtroTags').then((response: any) => {
      component.datasource.updateData(response.data.dataset.filtroTags);
      this.dadosOriginal[component.name] = response.data.dataset.filtroTags;
    });
  }

  public setTarefa({ component }:any) {
    const tarefas = Metadata.getInstance<SelectMultiple>('selectTagTarefas');
    if (component.value && component.value.length > 0) {
      tarefas.datasource.updateData(this.dadosOriginal[tarefas.name]);
      const copia = tarefas.datasource as any;
      const novoDataSource = (copia.allData || copia.data).filter((elemento: any) => component.value.includes(elemento.NRATIVIDADE));
      tarefas.datasource.updateData(novoDataSource);
      tarefas.setValue(tarefas.value.filter((elemento: any) => (novoDataSource.map((nrs: any) => nrs.NRTAREFA).includes(elemento))));
      tarefas.disabled = false;
    } else {
      tarefas.disabled = true;
      tarefas.setValue([]);
    }
  }

  public async gerarCriterio() {
    const intensValid = Metadata.getInstance<SelectMultiple>('selectItens').validate();
    const tarefas = Metadata.getInstance<SelectMultiple>('selectTagTarefas');
    const tarefasValid = tarefas.validate();

    if (intensValid && tarefasValid) {
      Metadata.getInstance<Button>('linhaBotaoGerarCriterios').isVisible = false;
      Metadata.getInstance<Button>('linhaBotaoSalvarFormulario').isVisible = true;
      const iterablePrincipal = Metadata.getInstance<IterableComponentRender>('IterableCardsTag');
      await Http.post('/getGrupo', tarefas.value.map((elemento: any) => elemento.NRTAREFA)).then((response: any) => {
        iterablePrincipal.datasource.updateData(response.data.dataset.grupos);
      }).then(() => {
        Metadata.getInstance<Row>('LinhaComLegendas').isVisible = true;
        iterablePrincipal.isVisible = true;
      });
    }
  }

  public btnGerarCriterio() {
    Metadata.getInstance<Button>('linhaBotaoGerarCriterios').isVisible = true;
    Metadata.getInstance<Button>('linhaBotaoSalvarFormulario').isVisible = false;
    Metadata.getInstance<IterableComponentRender>('IterableCardsTag').isVisible = false;
    Metadata.getInstance<Row>('LinhaComLegendas').isVisible = false;
  }

  public getDadosDatasource({ component }: any) {
    const dados = component.name.split('_');
    const parentIterable = Metadata.getInstance<IterableComponentRender>(dados[1].replaceAll('-', '_'));
    const parentIndex = parentIterable.datasource.data.findIndex((row) => row[dados[3]] === Number(dados[2]));
    component.datasource.data = parentIterable.datasource.data[parentIndex].SUBLIST;
  }

  public clickChip({ component }: any) {
    component.outlined = !component.outlined;
  }

  public getChip() {
    const dadosIterable = Metadata.getInstance<IterableComponentRender>('IterableCardsTag').datasource.data;
    const chips: any = [];
    dadosIterable.forEach((elemento: any) => {
      elemento.SUBLIST.forEach((sub: any) => {
        sub.SUBLIST.forEach((card: any) => {
          const chip = Metadata.getInstance<Chip>(`${elemento.DSTAGSTAREFA}_${sub.NRTAREFA}_${card.NRCRITERIOTAREFA}`);
          if (!chip.outlined) {
            chips.push({
              NRTAGSTAREFA: elemento.NRTAGSTAREFA,
              NRTAREFA: sub.NRTAREFA,
              NRCRITERIOTAREFA: card.NRCRITERIOTAREFA,
            });
          }
        });
      });
    });
    return chips;
  }

  public salvar({ component }: any) {
    this.getChip();
    const form = Metadata.getInstance<Form>('FormPrincipal');
    const chips = this.getChip();
    if (form.validate() && chips.length > 0) {
      LoadingService.show((component.label === 'Salvar') ? 'Salvando ...' : 'Atualizando ...');
      Http.post((component.label === 'Salvar') ? '/insertChecklist' : '/atualizarChecklist', {
        NRCHECKLIST: window.location.search.slice(1),
        tipoLocal: Metadata.getInstance<Select>('selectLocal').value,
        nomeCheckList: Metadata.getInstance<TextInput>('nomeCheckList').value,
        descricaoChecklist: Metadata.getInstance<TextInput>('descricaoChecklist').value,
        itensChecklist: Metadata.getInstance<SelectMultiple>('selectItens').value,
        selectTagTarefas: Metadata.getInstance<SelectMultiple>('selectTagTarefas').value,
        dadosChip: chips,
      }).then((resposta: any) => {
        if (resposta.data.dataset.resposta[0] === 'OK') {
          LoadingService.hide();
          this.mensagemSucesso(`Checklist ${(component.label === 'Salvar') ? 'salvo' : 'atualizado'} com sucesso!`);
          Router.replace('/checklist');
        } else {
          LoadingService.hide();
          this.mensagemErro(`ao ${(component.label === 'Salvar') ? 'salvar' : 'atualizar'}`);
        }
      });
    } else if (chips.length < 0) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Favor preencher os campos obrigatorios(*)!',
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: 'Favor selecionar pelo menos um critério.',
        top: true,
        left: false,
        timeout: 3000,
        color: 'error',
      });
    }
  }

  public mensagemErro(msg: string) {
    DialogService.show({
      name: 'Dialog',
      title: 'Ops! Ocorreu um erro',
      text: `Ocorreu um erro ${msg}.<br>Tente novamente mais tarde!<br>Se o erro persistir, contacte o administrador do sistema.`,
    });
  }

  public mensagemSucesso(msg: string) {
    AlertService.show({
      name: 'Alerta',
      text: msg,
      top: true,
      timeout: 1500,
      color: 'success',
    });
  }

  public async edicaoChecklist() {
    if (window.location.search.slice(1) !== '' && window.location.search.slice(1) !== null) {
      const btnCriar = Metadata.getInstance<Button>('botaoCriar');
      btnCriar.label = 'Atualizar';
      let dadosChip: any = [];
      let tarefas: any = [];
      let NRATIVIDADE: any = [];
      await this.getTipoLocal({ component: Metadata.getInstance<Select>('selectLocal') });
      await this.getItens({ component: Metadata.getInstance<SelectMultiple>('selectItens') });
      await this.getTarefa({ component: Metadata.getInstance<SelectMultiple>('selectTagTarefas') });
      await Http.get(`/detalhesCheckist?NRCHECKLIST=${window.location.search.slice(1)}`).then((resposta) => {
        const dadosBanco = resposta.data.dataset.resposta;
        Metadata.getInstance<Select>('selectLocal').setValue(dadosBanco.NRTIPOLOCALCLI);
        Metadata.getInstance<TextInput>('nomeCheckList').value = dadosBanco.NMCHECKLIST;
        Metadata.getInstance<TextInput>('descricaoChecklist').value = dadosBanco.DSCHECKLIST;
        NRATIVIDADE = dadosBanco.NRATIVIDADE.filter((elemento: any, pos: number) => dadosBanco.NRATIVIDADE.indexOf(elemento) === pos);
        Metadata.getInstance<SelectMultiple>('selectItens').setValue(NRATIVIDADE.map((elemento: any) => Number(elemento)));
        dadosChip = dadosBanco.CRITERIOS;
        tarefas = dadosBanco.NRTAREFA.filter((elemento: any, pos: number) => dadosBanco.NRTAREFA.indexOf(elemento) === pos);
      });
      await Metadata.getInstance<SelectMultiple>('selectTagTarefas').setValue(tarefas);
      await this.gerarCriterio();
      this.marqueChip(dadosChip);
    }
  }

  public marqueChip(dados: any) {
    dados.forEach((elemento: any) => {
      const chip = Metadata.getInstance<Chip>(elemento);
      chip.outlined = false;
    });
  }
}
